@@ -35,6 +35,7 @@ import org.apache.flink.connector.kafka.dynamic.source.split.DynamicKafkaSourceS
 import org.apache.flink.connector.kafka.source.KafkaPropertiesUtil;
 import org.apache.flink.connector.kafka.source.enumerator.KafkaSourceEnumState;
 import org.apache.flink.connector.kafka.source.enumerator.KafkaSourceEnumerator;
+import org.apache.flink.connector.kafka.source.enumerator.TopicPartitionAndAssignmentStatus;
 import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
 import org.apache.flink.connector.kafka.source.enumerator.subscriber.KafkaSubscriber;
 import org.apache.flink.connector.kafka.source.split.KafkaPartitionSplit;
@@ -288,23 +289,26 @@ public class DynamicKafkaSourceEnumerator
 
         // create enumerators
         for (Entry<String, Set<String>> activeClusterTopics : latestClusterTopicsMap.entrySet()) {
-            final Set<TopicPartition> activeTopicPartitions = new HashSet<>();
-
-            if (dynamicKafkaSourceEnumState
+            KafkaSourceEnumState kafkaSourceEnumState =
+                    dynamicKafkaSourceEnumState
                             .getClusterEnumeratorStates()
-                            .get(activeClusterTopics.getKey())
-                    != null) {
-                Set<TopicPartition> oldTopicPartitions =
-                        dynamicKafkaSourceEnumState
-                                .getClusterEnumeratorStates()
-                                .get(activeClusterTopics.getKey())
-                                .assignedPartitions();
+                            .get(activeClusterTopics.getKey());
+
+            final KafkaSourceEnumState newKafkaSourceEnumState;
+            if (kafkaSourceEnumState != null) {
+                final Set<String> activeTopics = activeClusterTopics.getValue();
+
                 // filter out removed topics
-                for (TopicPartition oldTopicPartition : oldTopicPartitions) {
-                    if (activeClusterTopics.getValue().contains(oldTopicPartition.topic())) {
-                        activeTopicPartitions.add(oldTopicPartition);
-                    }
-                }
+                Set<TopicPartitionAndAssignmentStatus> partitions =
+                        kafkaSourceEnumState.partitions().stream()
+                                .filter(tp -> activeTopics.contains(tp.topicPartition().topic()))
+                                .collect(Collectors.toSet());
+
+                newKafkaSourceEnumState =
+                        new KafkaSourceEnumState(
+                                partitions, kafkaSourceEnumState.initialDiscoveryFinished());
+            } else {
+                newKafkaSourceEnumState = new KafkaSourceEnumState(Collections.emptySet(), false);
             }
 
             // restarts enumerator from state using only the active topic partitions, to avoid
@@ -312,12 +316,7 @@ public class DynamicKafkaSourceEnumerator
             createEnumeratorWithAssignedTopicPartitions(
                     activeClusterTopics.getKey(),
                     activeClusterTopics.getValue(),
-                    dynamicKafkaSourceEnumState
-                            .getClusterEnumeratorStates()
-                            .getOrDefault(
-                                    activeClusterTopics.getKey(),
-                                    new KafkaSourceEnumState(
-                                            Collections.emptySet(), Collections.emptySet(), false)),
+                    newKafkaSourceEnumState,
                     clusterProperties.get(activeClusterTopics.getKey()));
         }
 
