@@ -23,8 +23,8 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.CoreOptions;
 import org.apache.flink.core.execution.JobClient;
 import org.apache.flink.core.execution.SavepointFormatType;
-import org.apache.flink.core.testutils.FlinkAssertions;
 import org.apache.flink.runtime.jobgraph.SavepointConfigOptions;
+import org.apache.flink.runtime.messages.FlinkJobTerminatedWithoutCancellationException;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.streaming.api.functions.sink.SinkFunction;
@@ -36,11 +36,14 @@ import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.utils.EncodingUtils;
 import org.apache.flink.test.util.SuccessException;
 import org.apache.flink.types.Row;
+import org.apache.flink.util.function.RunnableWithException;
 
 import org.apache.kafka.clients.consumer.NoOffsetForPartitionException;
 import org.apache.kafka.clients.consumer.OffsetAndMetadata;
 import org.apache.kafka.common.TopicPartition;
+import org.apache.kafka.common.errors.UnknownTopicOrPartitionException;
 import org.assertj.core.api.Assertions;
+import org.assertj.core.api.ThrowingConsumer;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -54,6 +57,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
@@ -64,6 +68,7 @@ import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
 import static org.apache.flink.core.testutils.CommonTestUtils.waitUtil;
+import static org.apache.flink.core.testutils.FlinkAssertions.anyCauseMatches;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaTableTestUtils.collectAllRows;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaTableTestUtils.collectRows;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaTableTestUtils.readLines;
@@ -185,7 +190,227 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
+    }
+
+    @Test
+    public void testKafkaSourceSinkWithTopicList() throws Exception {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String topic1 = "topics1_" + format + "_" + UUID.randomUUID();
+        final String topic2 = "topics2_" + format + "_" + UUID.randomUUID();
+        createTestTopic(topic2, 1, 1);
+        createTestTopic(topic1, 1, 1);
+
+        // ---------- Produce an event time stream into Kafka -------------------
+        String groupId = getStandardProps().getProperty("group.id");
+        String bootstraps = getBootstrapServers();
+        final String createTableTemplate =
+                "CREATE TABLE %s (\n"
+                        + "  `topic` STRING METADATA,\n"
+                        + "  `user_id` INT,\n"
+                        + "  `item_id` INT,\n"
+                        + "  `behavior` STRING\n"
+                        + ") WITH (\n"
+                        + "  'connector' = '%s',\n"
+                        + "  'topic' = '%s',\n"
+                        + "  'properties.bootstrap.servers' = '%s',\n"
+                        + "  'properties.group.id' = '%s',\n"
+                        + "  'scan.startup.mode' = 'earliest-offset',\n"
+                        + "  'scan.bounded.mode' = 'latest-offset',\n"
+                        + "  %s\n"
+                        + ")\n";
+        final String createTopicListTable =
+                String.format(
+                        createTableTemplate,
+                        "kafka",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        String.join(";", Arrays.asList(topic1, topic2)),
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+        final String createTopic1Table =
+                String.format(
+                        createTableTemplate,
+                        "topic1",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topic1,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+        final String createTopic2Table =
+                String.format(
+                        createTableTemplate,
+                        "topic2",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topic2,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+
+        tEnv.executeSql(createTopicListTable);
+        tEnv.executeSql(createTopic1Table);
+        tEnv.executeSql(createTopic2Table);
+
+        List<Row> values =
+                Arrays.asList(
+                        Row.of(topic1, 1, 1102, "behavior 1"),
+                        Row.of(topic2, 2, 1103, "behavior 2"));
+        tEnv.fromValues(values).insertInto("kafka").execute().await();
+
+        // ---------- Consume stream from Kafka -------------------
+        List<Row> results = collectAllRows(tEnv.sqlQuery("SELECT * from kafka"));
+        List<Row> topic1Results = collectAllRows(tEnv.sqlQuery("SELECT * from topic1"));
+        List<Row> topic2Results = collectAllRows(tEnv.sqlQuery("SELECT * from topic2"));
+        assertThat(results)
+                .containsExactlyInAnyOrder(
+                        Row.of(topic1, 1, 1102, "behavior 1"),
+                        Row.of(topic2, 2, 1103, "behavior 2"));
+        assertThat(topic1Results).containsExactly(Row.of(topic1, 1, 1102, "behavior 1"));
+        assertThat(topic2Results).containsExactly(Row.of(topic2, 2, 1103, "behavior 2"));
+
+        // ------------- cleanup -------------------
+        cleanupTopic(topic1);
+        cleanupTopic(topic2);
+    }
+
+    @Test
+    public void testKafkaSourceSinkWithTopicPattern() throws Exception {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String topic1 = "topics1_" + format + "_" + UUID.randomUUID();
+        final String topic2 = "topics2_" + format + "_" + UUID.randomUUID();
+        final String topicPattern = "topics.*";
+        createTestTopic(topic2, 1, 1);
+        createTestTopic(topic1, 1, 1);
+
+        // ---------- Produce an event time stream into Kafka -------------------
+        String groupId = getStandardProps().getProperty("group.id");
+        String bootstraps = getBootstrapServers();
+        final String createTableTemplate =
+                "CREATE TABLE %s (\n"
+                        + "  `topic` STRING METADATA,\n"
+                        + "  `user_id` INT,\n"
+                        + "  `item_id` INT,\n"
+                        + "  `behavior` STRING\n"
+                        + ") WITH (\n"
+                        + "  'connector' = '%s',\n"
+                        + "  'topic-pattern' = '%s',\n"
+                        + "  'properties.bootstrap.servers' = '%s',\n"
+                        + "  'properties.group.id' = '%s',\n"
+                        + "  'scan.startup.mode' = 'earliest-offset',\n"
+                        + "  'scan.bounded.mode' = 'latest-offset',\n"
+                        + "  %s\n"
+                        + ")\n";
+        final String createTopicPatternTable =
+                String.format(
+                        createTableTemplate,
+                        "kafka",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topicPattern,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+        final String createTopic1Table =
+                String.format(
+                        createTableTemplate,
+                        "topic1",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topic1,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+        final String createTopic2Table =
+                String.format(
+                        createTableTemplate,
+                        "topic2",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topic2,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+
+        tEnv.executeSql(createTopicPatternTable);
+        tEnv.executeSql(createTopic1Table);
+        tEnv.executeSql(createTopic2Table);
+
+        List<Row> values =
+                Arrays.asList(
+                        Row.of(topic1, 1, 1102, "behavior 1"),
+                        Row.of(topic2, 2, 1103, "behavior 2"));
+        tEnv.fromValues(values).insertInto("kafka").execute().await();
+
+        // ---------- Consume stream from Kafka -------------------
+        List<Row> results = collectAllRows(tEnv.sqlQuery("SELECT * from kafka"));
+        List<Row> topic1Results = collectAllRows(tEnv.sqlQuery("SELECT * from topic1"));
+        List<Row> topic2Results = collectAllRows(tEnv.sqlQuery("SELECT * from topic2"));
+        assertThat(results)
+                .containsExactlyInAnyOrder(
+                        Row.of(topic1, 1, 1102, "behavior 1"),
+                        Row.of(topic2, 2, 1103, "behavior 2"));
+        assertThat(topic1Results).containsExactly(Row.of(topic1, 1, 1102, "behavior 1"));
+        assertThat(topic2Results).containsExactly(Row.of(topic2, 2, 1103, "behavior 2"));
+
+        // ------------- cleanup -------------------
+
+        cleanupTopic(topic1);
+        cleanupTopic(topic2);
+    }
+
+    @Test
+    public void testKafkaSourceEmptyResultOnDeletedOffsets() throws Exception {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String topic = "bounded_" + format + "_" + UUID.randomUUID();
+        createTestTopic(topic, 1, 1);
+        // ---------- Produce an event time stream into Kafka -------------------
+        String groupId = getStandardProps().getProperty("group.id");
+        String bootstraps = getBootstrapServers();
+
+        final String createTable =
+                String.format(
+                        "CREATE TABLE kafka (\n"
+                                + "  `user_id` INT,\n"
+                                + "  `item_id` INT,\n"
+                                + "  `behavior` STRING\n"
+                                + ") WITH (\n"
+                                + "  'connector' = '%s',\n"
+                                + "  'topic' = '%s',\n"
+                                + "  'properties.bootstrap.servers' = '%s',\n"
+                                + "  'properties.group.id' = '%s',\n"
+                                + "  'scan.startup.mode' = 'specific-offsets',\n"
+                                + "  'scan.bounded.mode' = 'specific-offsets',\n"
+                                + "  'scan.startup.specific-offsets' = 'partition:0,offset:1',\n"
+                                + "  'scan.bounded.specific-offsets' = 'partition:0,offset:3',\n"
+                                + "  %s\n"
+                                + ")\n",
+                        KafkaDynamicTableFactory.IDENTIFIER,
+                        topic,
+                        bootstraps,
+                        groupId,
+                        formatOptions());
+        tEnv.executeSql(createTable);
+        List<Row> values =
+                Arrays.asList(
+                        Row.of(1, 1102, "behavior 1"),
+                        Row.of(2, 1103, "behavior 2"),
+                        Row.of(3, 1104, "behavior 3"));
+        tEnv.fromValues(values).insertInto("kafka").execute().await();
+        // ---------- Delete events from Kafka -------------------
+        Map<Integer, Long> partitionOffsetsToDelete = new HashMap<>();
+        partitionOffsetsToDelete.put(0, 3L);
+        deleteRecords(topic, partitionOffsetsToDelete);
+        // ---------- Consume stream from Kafka -------------------
+        List<Row> results = new ArrayList<>();
+        env = StreamExecutionEnvironment.getExecutionEnvironment();
+        tEnv = StreamTableEnvironment.create(env);
+        tEnv.executeSql(createTable);
+        results.addAll(collectAllRows(tEnv.sqlQuery("SELECT * FROM kafka")));
+        assertThat(results).isEmpty();
+
+        // ------------- cleanup -------------------
+
+        cleanupTopic(topic);
     }
 
     @Test
@@ -239,7 +464,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -296,7 +521,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -481,7 +706,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -562,7 +787,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -640,7 +865,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(topic);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -756,8 +981,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        deleteTestTopic(orderTopic);
-        deleteTestTopic(productTopic);
+        cleanupTopic(orderTopic);
+        cleanupTopic(productTopic);
     }
 
     private void initialProductChangelog(String topic, String bootstraps) throws Exception {
@@ -872,8 +1097,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        tableResult.getJobClient().ifPresent(JobClient::cancel);
-        deleteTestTopic(topic);
+        cancelJob(tableResult);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -947,8 +1172,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        tableResult.getJobClient().ifPresent(JobClient::cancel);
-        deleteTestTopic(topic);
+        cancelJob(tableResult);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -1079,8 +1304,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
 
         // ------------- cleanup -------------------
 
-        tableResult.getJobClient().ifPresent(JobClient::cancel);
-        deleteTestTopic(topic);
+        cancelJob(tableResult);
+        cleanupTopic(topic);
     }
 
     @Test
@@ -1096,7 +1321,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
     @Test
     public void testStartFromGroupOffsetsNone() {
         Assertions.assertThatThrownBy(() -> testStartFromGroupOffsetsWithNoneResetStrategy())
-                .satisfies(FlinkAssertions.anyCauseMatches(NoOffsetForPartitionException.class));
+                .satisfies(anyCauseMatches(NoOffsetForPartitionException.class));
     }
 
     private List<String> appendNewData(
@@ -1212,10 +1437,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
             KafkaTableTestUtils.waitingExpectedResults(sinkName, expected, Duration.ofSeconds(15));
         } finally {
             // ------------- cleanup -------------------
-            if (tableResult != null) {
-                tableResult.getJobClient().ifPresent(JobClient::cancel);
-            }
-            deleteTestTopic(topic);
+            cancelJob(tableResult);
+            cleanupTopic(topic);
         }
     }
 
@@ -1234,10 +1457,8 @@ public class KafkaTableITCase extends KafkaTableTestBase {
             tableResult.await();
         } finally {
             // ------------- cleanup -------------------
-            if (tableResult != null) {
-                tableResult.getJobClient().ifPresent(JobClient::cancel);
-            }
-            deleteTestTopic(topic);
+            cancelJob(tableResult);
+            cleanupTopic(topic);
         }
     }
 
@@ -1293,4 +1514,33 @@ public class KafkaTableITCase extends KafkaTableTestBase {
             return false;
         }
     }
+
+    private void cleanupTopic(String topic) {
+        ignoreExceptions(
+                () -> deleteTestTopic(topic),
+                anyCauseMatches(UnknownTopicOrPartitionException.class));
+    }
+
+    private static void cancelJob(TableResult tableResult) {
+        if (tableResult != null && tableResult.getJobClient().isPresent()) {
+            ignoreExceptions(
+                    () -> tableResult.getJobClient().get().cancel().get(),
+                    anyCauseMatches(FlinkJobTerminatedWithoutCancellationException.class),
+                    anyCauseMatches(
+                            "MiniCluster is not yet running or has already been shut down."));
+        }
+    }
+
+    @SafeVarargs
+    private static void ignoreExceptions(
+            RunnableWithException runnable, ThrowingConsumer<? super Throwable>... ignoreIf) {
+        try {
+            runnable.run();
+        } catch (InterruptedException e) {
+            Thread.currentThread().interrupt();
+        } catch (Exception ex) {
+            // check if the exception is one of the ignored ones
+            assertThat(ex).satisfiesAnyOf(ignoreIf);
+        }
+    }
 }
